# pylint: disable=too-many-locals,logging-format-interpolation
import logging
import os
from typing import Any
from django.core.management.base import CommandParser
from django.db import transaction
from django.utils.timezone import now
from jutil.admin import admin_log
from jutil.command import SafeCommand
from jsanctions.models import EuCombinedSanctionsList, SanctionEntity, RegulationSummary, BirthDate, Identification, \
    Remark, Address, Citizenship, NameAlias, Regulation, SubjectType
from jutil.format import camel_case_to_underscore

logger = logging.getLogger(__name__)


def eu_set_object_attr(obj, k: str, v, max_length: int = 512):
    if v and isinstance(v, str) and len(v) > max_length:
        logger.warning('{key} value truncated to {max_length} characters. Original was: "{original}"'.format(
            key=k, original=v, max_length=max_length))
        v = v[:max_length-3] + '...'
    setattr(obj, k, v)


def eu_set_simple_members(obj, data: dict, commit: bool = True, verbose: bool = True, padding: int = 0, **kwargs):  # noqa
    class_map = {
        'regulationSummary': RegulationSummary,
        'subjectType': SubjectType,
    }
    array_class_map = {
        'birthdate': BirthDate,
        'identification': Identification,
        'address': Address,
        'citizenship': Citizenship,
        'nameAlias': NameAlias,
        'regulation': Regulation,
    }

    padding_str = ' ' * padding
    obj.save()
    obj2: Any
    for k0, v0 in data.items():
        if k0[0] == '@':
            k = camel_case_to_underscore(k0[1:])
            if k == 'birthdate':
                k = 'birth_date'  # special case because class name same as attribute
            if hasattr(obj, k):
                eu_set_object_attr(obj, k, v0)
                if verbose:
                    logger.info('{}{}: {} = {}'.format(padding_str, obj, k, v0))
        elif k0 in class_map:
            k = camel_case_to_underscore(k0)
            if hasattr(obj, k):
                if k == 'subject_type':
                    obj2 = SubjectType.objects.get_or_create(
                        code=v0.get('@code', ''),
                        classification_code=v0.get('@classificationCode', ''))[0]
                elif k == 'regulation_summary':
                    obj2 = RegulationSummary.objects.get_or_create(
                        regulation_type=v0.get('@regulationType', ''),
                        publication_date=v0.get('@publicationDate', None),
                        publication_url=v0.get('@publicationUrl', ''),
                        number_title=v0.get('@numberTitle', ''))[0]
                else:
                    obj2 = class_map[k0]()
                    kwargs2 = {}
                    kwargs2[k] = obj2
                    for k2, v2 in kwargs.items():
                        eu_set_object_attr(obj2, k2, v2)
                        kwargs2[k2] = v2

                    eu_set_simple_members(obj2, v0, verbose=verbose, padding=padding+4, **kwargs2)

                eu_set_object_attr(obj, k, obj2)
        elif k0 in array_class_map:
            k = camel_case_to_underscore(k0)
            cls = array_class_map[k0]
            for v0_data in v0:
                obj2 = cls()
                kwargs2 = {}
                kwargs2[k] = obj2
                for k2, v2 in kwargs.items():
                    eu_set_object_attr(obj2, k2, v2)
                    kwargs2[k2] = v2
                obj2.clean()
                obj2.save()
                eu_set_simple_members(obj2, v0_data, verbose=verbose, padding=padding+4, **kwargs2)
        elif k0 == 'remark':
            for v0_str in v0:
                Remark.objects.create(text=v0_str, container=obj)

    if commit:
        if verbose:
            logger.info('Saving {}'.format(obj))
        obj.clean()
        obj.save()


def create_eu_sanctions(source: EuCombinedSanctionsList, verbose: bool = True):
    data = source.load_xml_as_dict()
    # if verbose:
    #     pprint(data)
    eu_set_simple_members(source, data, verbose=verbose)
    entities_list = data.get('sanctionEntity', [])
    logger.info('Importing {} sanction entities from {}'.format(len(entities_list), os.path.basename(source.file.name)))
    t0 = now()
    for se_data in entities_list:
        assert isinstance(se_data, dict)
        if verbose:
            logger.info('  sanctionEntity')
        with transaction.atomic():
            se = SanctionEntity.objects.create(source=source)
            eu_set_simple_members(se, se_data, verbose=verbose, padding=4, sanction=se)
    source.imported = now()
    source.save()
    msg = 'Imported {} sanction entities from {} in {}'.format(len(entities_list), os.path.basename(source.file.name), source.imported - t0)
    logger.info(msg)
    admin_log([source], msg)


class Command(SafeCommand):
    help = "Imports EU combined sanction lists and saves results to the DB"

    def add_arguments(self, parser: CommandParser):
        parser.add_argument('--url', type=str)
        parser.add_argument('--file', type=str)
        parser.add_argument('--delete-old', action='store_true')
        parser.add_argument('--source', type=int)
        parser.add_argument('--new', action='store_true')
        parser.add_argument('--verbose', action='store_true')

    def do(self, *args, **options):
        verbose = options['verbose']
        source = None
        if options['url']:
            filename = options['file'] if options['file'] else 'EU-combined-{}.xml'.format(now().date().isoformat())
            source = EuCombinedSanctionsList.objects.create_from_url(options['url'], filename)
        elif options['file']:
            source = EuCombinedSanctionsList.objects.create_from_filename(options['file'])
        elif options['source']:
            source = EuCombinedSanctionsList.objects.get(id=options['source'])
        elif options['new']:
            source = EuCombinedSanctionsList.objects.filter(imported=None).order_by('id').first()

        if not source:
            print('Nothing to import')
            return

        if options['delete_old']:
            for e in EuCombinedSanctionsList.objects.all().exclude(id=source.id):
                logger.info('Deleting {}'.format(e))
                e.delete()

        assert isinstance(source, EuCombinedSanctionsList)
        create_eu_sanctions(source, verbose=verbose)



from typing import Dict, Any


def dict_filter_attributes(data: Dict[str, Any], fn) -> Dict[str, Any]:
    if isinstance(data, dict):
        for k, v in list(data.items()):
            if isinstance(v, dict):
                v = dict_filter_attributes(v, fn)
            elif isinstance(v, list):
                v = [dict_filter_attributes(e, fn) for e in v]
            elif k.startswith('@') and len(k) > 1 and isinstance(v, str):
                v = fn(k, v)
            data[k] = v
    return data



import os
from django.conf import settings
from django.test import TestCase
from jsanctions.management.commands.import_eu_sanctions import create_eu_sanctions
from jsanctions.models import EuCombinedSanctionsList, SanctionEntity


class Tests(TestCase):
    def test_eu_sanctions_import(self):
        print('test_eu_sanctions_import')
        filename = os.path.join(settings.BASE_DIR, 'data/eu/20190827-FULL-1_1.xml')
        source = EuCombinedSanctionsList.objects.create_from_filename(filename)
        assert isinstance(source, EuCombinedSanctionsList)
        create_eu_sanctions(source, verbose=False)
        self.assertEqual(SanctionEntity.objects.all().count(), 2200)



# Generated by Django 2.2.5 on 2019-09-12 04:44

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='SanctionListObject',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
        ),
        migrations.CreateModel(
            name='RegulationSummary',
            fields=[
                ('sanctionlistobject_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='jsanctions.SanctionListObject')),
                ('regulation_type', models.CharField(blank=True, default='', max_length=32, verbose_name='regulation type')),
                ('number_title', models.CharField(blank=True, default='', max_length=256, verbose_name='number title')),
                ('publication_date', models.DateField(blank=True, default=None, null=True, verbose_name='publication date')),
                ('publication_url', models.URLField(blank=True, default='', verbose_name='url')),
            ],
            options={
                'verbose_name': 'regulation summary',
                'verbose_name_plural': 'regulation summaries',
            },
            bases=('jsanctions.sanctionlistobject',),
        ),
        migrations.CreateModel(
            name='SanctionsListFile',
            fields=[
                ('sanctionlistobject_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='jsanctions.SanctionListObject')),
                ('created', models.DateTimeField(blank=True, db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('imported', models.DateTimeField(blank=True, db_index=True, default=None, editable=False, null=True, verbose_name='imported')),
                ('generation_date', models.DateField(blank=True, db_index=True, default=None, editable=False, null=True, verbose_name='generation date')),
                ('file', models.FileField(upload_to='uploads', verbose_name='file')),
            ],
            options={
                'verbose_name': 'sanction list',
                'verbose_name_plural': 'sanction lists',
            },
            bases=('jsanctions.sanctionlistobject',),
        ),
        migrations.CreateModel(
            name='SubjectType',
            fields=[
                ('sanctionlistobject_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='jsanctions.SanctionListObject')),
                ('code', models.CharField(blank=True, default='', max_length=256, verbose_name='code')),
                ('classification_code', models.CharField(blank=True, default='', max_length=32, verbose_name='classification code')),
            ],
            options={
                'verbose_name': 'subject type',
                'verbose_name_plural': 'subject types',
            },
            bases=('jsanctions.sanctionlistobject',),
        ),
        migrations.CreateModel(
            name='Remark',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.TextField(blank=True, verbose_name='text')),
                ('container', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='jsanctions.SanctionListObject')),
            ],
            options={
                'verbose_name': 'remark',
                'verbose_name_plural': 'remarks',
            },
        ),
        migrations.CreateModel(
            name='EuCombinedSanctionsList',
            fields=[
                ('sanctionslistfile_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='jsanctions.SanctionsListFile')),
                ('global_file_id', models.CharField(blank=True, default='', max_length=256, verbose_name='global file id')),
            ],
            options={
                'verbose_name': 'EU combined sanction list',
                'verbose_name_plural': 'EU combined sanction lists',
            },
            bases=('jsanctions.sanctionslistfile',),
        ),
        migrations.CreateModel(
            name='SanctionEntity',
            fields=[
                ('sanctionlistobject_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='jsanctions.SanctionListObject')),
                ('designation_details', models.CharField(blank=True, default='', max_length=256, verbose_name='designation details')),
                ('united_nation_id', models.CharField(blank=True, default='', max_length=256, verbose_name='United Nation identifier')),
                ('eu_reference_number', models.CharField(blank=True, default='', max_length=256, verbose_name='EU reference number')),
                ('logical_id', models.BigIntegerField(blank=True, default=None, null=True, verbose_name='logical id')),
                ('subject_type', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.PROTECT, to='jsanctions.SubjectType', verbose_name='subject type')),
                ('source', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='jsanctions.EuCombinedSanctionsList', verbose_name='source')),
            ],
            options={
                'verbose_name': 'sanction entity',
                'verbose_name_plural': 'sanction entities',
            },
            bases=('jsanctions.sanctionlistobject',),
        ),
        migrations.CreateModel(
            name='Regulation',
            fields=[
                ('sanctionlistobject_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='jsanctions.SanctionListObject')),
                ('regulation_type', models.CharField(blank=True, default='', max_length=256, verbose_name='regulation type')),
                ('organisation_type', models.CharField(blank=True, default='', max_length=256, verbose_name='organization type')),
                ('publication_date', models.DateField(blank=True, default=None, null=True, verbose_name='publication date')),
                ('publication_url', models.URLField(blank=True, default='', verbose_name='url')),
                ('entry_into_force_date', models.DateField(blank=True, default=None, null=True, verbose_name='entry into force date')),
                ('number_title', models.CharField(blank=True, default='', max_length=256, verbose_name='number title')),
                ('programme', models.CharField(blank=True, default='', max_length=256, verbose_name='programmer')),
                ('logical_id', models.BigIntegerField(blank=True, default=None, null=True, verbose_name='logical id')),
                ('sanction', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='jsanctions.SanctionEntity', verbose_name='sanction entity')),
            ],
            options={
                'verbose_name': 'regulation',
                'verbose_name_plural': 'regulations',
            },
            bases=('jsanctions.sanctionlistobject',),
        ),
        migrations.CreateModel(
            name='NameAlias',
            fields=[
                ('sanctionlistobject_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='jsanctions.SanctionListObject')),
                ('first_name', models.CharField(blank=True, default='', max_length=256, verbose_name='first name')),
                ('middle_name', models.CharField(blank=True, default='', max_length=256, verbose_name='middle name')),
                ('last_name', models.CharField(blank=True, default='', max_length=256, verbose_name='last name')),
                ('whole_name', models.CharField(blank=True, default='', max_length=256, verbose_name='whole name')),
                ('name_language', models.CharField(blank=True, default='', max_length=5, verbose_name='name language')),
                ('function', models.CharField(blank=True, default='', max_length=256, verbose_name='function')),
                ('title', models.CharField(blank=True, default='', max_length=256, verbose_name='title')),
                ('regulation_language', models.CharField(blank=True, default='', max_length=5, verbose_name='regulation language')),
                ('logical_id', models.BigIntegerField(blank=True, default=None, null=True, verbose_name='logical id')),
                ('regulation_summary', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.PROTECT, to='jsanctions.RegulationSummary', verbose_name='regulation summary')),
                ('sanction', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='jsanctions.SanctionEntity', verbose_name='sanction entity')),
            ],
            options={
                'verbose_name': 'name alias',
                'verbose_name_plural': 'name aliases',
            },
            bases=('jsanctions.sanctionlistobject',),
        ),
        migrations.CreateModel(
            name='Identification',
            fields=[
                ('sanctionlistobject_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='jsanctions.SanctionListObject')),
                ('diplomatic', models.BooleanField(blank=True, default=None, null=True, verbose_name='diplomatic')),
                ('known_expired', models.BooleanField(blank=True, default=None, null=True, verbose_name='known expired')),
                ('known_false', models.BooleanField(blank=True, default=None, null=True, verbose_name='known false')),
                ('reported_lost', models.BooleanField(blank=True, default=None, null=True, verbose_name='reported lost')),
                ('revoked_by_issuer', models.BooleanField(blank=True, default=None, null=True, verbose_name='revoked by issuer')),
                ('issued_by', models.CharField(blank=True, default='', max_length=256, verbose_name='issued by')),
                ('latin_number', models.CharField(blank=True, default='', max_length=256, verbose_name='latin number')),
                ('name_on_document', models.CharField(blank=True, default='', max_length=256, verbose_name='name on document')),
                ('number', models.CharField(blank=True, default='', max_length=256, verbose_name='number')),
                ('region', models.CharField(blank=True, default='', max_length=256, verbose_name='region')),
                ('country_iso2_code', models.CharField(blank=True, default='', max_length=3, verbose_name='issued by')),
                ('country_description', models.CharField(blank=True, default='', max_length=256, verbose_name='country description')),
                ('identification_type_code', models.CharField(blank=True, default='', max_length=32, verbose_name='identification type code')),
                ('identification_type_description', models.CharField(blank=True, default='', max_length=256, verbose_name='identification type code')),
                ('regulation_language', models.CharField(blank=True, default='', max_length=5, verbose_name='regional language')),
                ('logical_id', models.BigIntegerField(blank=True, default=None, null=True, verbose_name='logical id')),
                ('regulation_summary', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.PROTECT, to='jsanctions.RegulationSummary', verbose_name='regulation summary')),
                ('sanction', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='jsanctions.SanctionEntity', verbose_name='sanction entity')),
            ],
            options={
                'verbose_name': 'identification',
                'verbose_name_plural': 'identifications',
            },
            bases=('jsanctions.sanctionlistobject',),
        ),
        migrations.CreateModel(
            name='Citizenship',
            fields=[
                ('sanctionlistobject_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='jsanctions.SanctionListObject')),
                ('region', models.CharField(blank=True, default='', max_length=256, verbose_name='region')),
                ('country_iso2_code', models.CharField(blank=True, default='', max_length=3, verbose_name='country')),
                ('country_description', models.CharField(blank=True, default='', max_length=256, verbose_name='country description')),
                ('regulation_language', models.CharField(blank=True, default='', max_length=5, verbose_name='regional language')),
                ('logical_id', models.BigIntegerField(blank=True, default=None, null=True, verbose_name='logical id')),
                ('regulation_summary', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.PROTECT, to='jsanctions.RegulationSummary', verbose_name='regulation summary')),
                ('sanction', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='jsanctions.SanctionEntity', verbose_name='sanction entity')),
            ],
            options={
                'verbose_name': 'citizenship',
                'verbose_name_plural': 'citizenships',
            },
            bases=('jsanctions.sanctionlistobject',),
        ),
        migrations.CreateModel(
            name='BirthDate',
            fields=[
                ('sanctionlistobject_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='jsanctions.SanctionListObject')),
                ('circa', models.BooleanField(blank=True, default=None, null=True, verbose_name='circa')),
                ('calendar_type', models.CharField(blank=True, default='', max_length=32, verbose_name='calendar type')),
                ('city', models.CharField(blank=True, default='', max_length=256, verbose_name='city')),
                ('zip_code', models.CharField(blank=True, default='', max_length=256, verbose_name='zip code')),
                ('birth_date', models.DateField(blank=True, default=None, null=True, verbose_name='birth date')),
                ('day_of_month', models.IntegerField(blank=True, default=None, null=True, verbose_name='day of month')),
                ('month_of_year', models.IntegerField(blank=True, default=None, null=True, verbose_name='month of year')),
                ('year', models.IntegerField(blank=True, default=None, null=True, verbose_name='year')),
                ('region', models.CharField(blank=True, default='', max_length=256, verbose_name='region')),
                ('place', models.CharField(blank=True, default='', max_length=256, verbose_name='place')),
                ('country_iso2_code', models.CharField(blank=True, default='', max_length=3, verbose_name='country')),
                ('country_description', models.CharField(blank=True, default='', max_length=256, verbose_name='country description')),
                ('regulation_language', models.CharField(blank=True, default='', max_length=5, verbose_name='regional language')),
                ('logical_id', models.BigIntegerField(blank=True, default=None, null=True, verbose_name='logical id')),
                ('sanction', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='jsanctions.SanctionEntity', verbose_name='sanction entity')),
            ],
            options={
                'verbose_name': 'birth date',
                'verbose_name_plural': 'birth dates',
            },
            bases=('jsanctions.sanctionlistobject',),
        ),
        migrations.CreateModel(
            name='Address',
            fields=[
                ('sanctionlistobject_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='jsanctions.SanctionListObject')),
                ('city', models.CharField(blank=True, default='', max_length=256, verbose_name='city')),
                ('street', models.CharField(blank=True, default='', max_length=256, verbose_name='street')),
                ('po_box', models.CharField(blank=True, default='', max_length=256, verbose_name='p.o. box')),
                ('zip_code', models.CharField(blank=True, default='', max_length=256, verbose_name='zip code')),
                ('as_at_listing_time', models.BooleanField(blank=True, default=None, null=True, verbose_name='as at listing time')),
                ('place', models.CharField(blank=True, default='', max_length=256, verbose_name='place')),
                ('region', models.CharField(blank=True, default='', max_length=256, verbose_name='region')),
                ('country_iso2_code', models.CharField(blank=True, default='', max_length=3, verbose_name='country')),
                ('country_description', models.CharField(blank=True, default='', max_length=256, verbose_name='country description')),
                ('regulation_language', models.CharField(blank=True, default='', max_length=5, verbose_name='regional language')),
                ('logical_id', models.BigIntegerField(blank=True, default=None, null=True, verbose_name='logical id')),
                ('regulation_summary', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.PROTECT, to='jsanctions.RegulationSummary', verbose_name='regulation summary')),
                ('sanction', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='jsanctions.SanctionEntity', verbose_name='sanction entity')),
            ],
            options={
                'verbose_name': 'address',
                'verbose_name_plural': 'addresses',
            },
            bases=('jsanctions.sanctionlistobject',),
        ),
    ]



from typing import Sequence
from django.contrib import admin
from django.contrib import messages
from django.utils.timezone import now
from django.utils.translation import gettext_lazy as _
from jutil.admin import ModelAdminBase, AdminFileDownloadMixin
from jsanctions.models import EuCombinedSanctionsList, SubjectType, Regulation, RegulationSummary, \
    Remark, NameAlias, SanctionEntity, BirthDate, Identification, Citizenship, Address


class SanctionsListAdminBase(ModelAdminBase):
    exclude = ()


class SubjectTypeAdmin(SanctionsListAdminBase):
    exclude = ()
    list_display = (
        'classification_code',
        'code',
    )


class SanctionsListAdmin(ModelAdminBase, AdminFileDownloadMixin):
    save_on_top = False

    list_display = [
        'created',
        'generation_date',
        'file',
        'imported',
    ]

    fields = [
        'created',
        'generation_date',
        'file',
        'imported',
    ]

    readonly_fields = [
        'created',
        'generation_date',
        'imported',
    ]

    raw_id_fields = (
    )

    date_hierarchy = 'created'

    def get_urls(self):
        return self.get_download_urls() + super().get_urls()


class EuCombinedSanctionsListAdmin(SanctionsListAdmin):
    fields = SanctionsListAdmin.fields + [
        'global_file_id',
    ]

    readonly_fields = SanctionsListAdmin.readonly_fields + [
        'global_file_id',
    ]


class RemarkAdmin(SanctionsListAdminBase):
    raw_id_fields = (
        'container',
    )
    list_display = (
        'id',
        'text_brief',
    )


class RemarkInlineAdmin(admin.TabularInline):
    model = Remark
    fk_name = 'container'
    extra = 0
    fields = (
        'text',
    )


class SanctionEntitySubFieldAdminBase(SanctionsListAdminBase):
    raw_id_fields: Sequence[str] = (
        'sanction',
    )


class SanctionEntityInlineAdmin(admin.StackedInline):
    fk_name = 'sanction'
    extra = 0


class NameAliasAdmin(SanctionEntitySubFieldAdminBase):
    raw_id_fields = (
        'sanction',
        'regulation_summary',
    )


class NameAliasInlineAdmin(SanctionEntityInlineAdmin):
    model = NameAlias
    fields = (
        'first_name',
        'middle_name',
        'last_name',
        'whole_name',
        'name_language',
        'function',
        'title',
        'regulation_language',
        'logical_id',
        'regulation_summary',
    )
    raw_id_fields = (
        'sanction',
        'regulation_summary',
    )


class BirthDateAdmin(SanctionEntitySubFieldAdminBase):
    list_display = (
        'id',
        'sanction',
        'birth_date',
    )
    search_fields = (
        'birth_date',
    )


class BirthDateInlineAdmin(SanctionEntityInlineAdmin):
    model = BirthDate
    fields = (
        'circa',
        'calendar_type',
        'city',
        'zip_code',
        'birth_date',
        'day_of_month',
        'month_of_year',
        'year',
        'region',
        'place',
        'country_iso2_code',
        'country_description',
        'regulation_language',
        'logical_id',
    )


class AddressAdmin(SanctionEntitySubFieldAdminBase):
    list_display = (
        'id',
        'sanction',
        'street',
        'city',
        'region',
        'country_iso2_code',
    )
    search_fields = (
        'street',
        'city',
        'region',
        'country_iso2_code',
    )
    list_filter = (
        'country_iso2_code',
    )


class AddressInlineAdmin(SanctionEntityInlineAdmin):
    model = Address
    fields = (
        'city',
        'street',
        'po_box',
        'zip_code',
        'as_at_listing_time',
        'place',
        'region',
        'country_iso2_code',
        'country_description',
        'regulation_language',
        'logical_id',
        'regulation_summary',
    )
    raw_id_fields = (
        'regulation_summary',
    )


class IdentificationAdmin(SanctionEntitySubFieldAdminBase):
    raw_id_fields = (
        'sanction',
        'regulation_summary',
    )


class IdentificationInlineAdmin(SanctionEntityInlineAdmin):
    model = Identification
    fields = (
        'diplomatic',
        'known_expired',
        'known_false',
        'reported_lost',
        'revoked_by_issuer',
        'issued_by',
        'latin_number',
        'name_on_document',
        'number',
        'region',
        'country_iso2_code',
        'country_description',
        'identification_type_code',
        'identification_type_description',
        'regulation_language',
        'logical_id',
        'regulation_summary',
    )
    raw_id_fields = (
        'regulation_summary',
    )


class CitizenshipAdmin(SanctionEntitySubFieldAdminBase):
    raw_id_fields = (
        'sanction',
        'regulation_summary',
    )


class CitizenshipInlineAdmin(SanctionEntityInlineAdmin):
    model = Citizenship
    fields = (
        'region',
        'country_iso2_code',
        'country_description',
        'regulation_language',
        'logical_id',
        'regulation_summary',
    )
    raw_id_fields = (
        'sanction',
        'regulation_summary',
    )


class RegulationInlineAdmin(SanctionEntityInlineAdmin):
    model = Regulation
    fields = (
        'regulation_type',
        'organisation_type',
        'publication_date',
        'publication_url',
        'entry_into_force_date',
        'number_title',
        'programme',
        'logical_id',
    )


class RegulationSummaryAdmin(SanctionsListAdminBase):
    exclude = ()
    date_hierarchy = 'publication_date'
    search_fields = (
        'number_title',
        'publication_date__year',
    )
    list_display = (
        'id',
        'regulation_type',
        'number_title',
        'publication_date',
        'publication_url',
    )


class DecadeBornListFilter(admin.SimpleListFilter):
    title = _('decade born')
    parameter_name = 'born'

    def lookups(self, request, model_admin):
        opts = [
            ('-1919', '-1919'),
        ]
        begin = 1920
        this_year = now().year
        while begin <= this_year:
            end = begin + 9
            if BirthDate.objects.filter(year__gte=begin, year__lte=end).first():
                value = '{}-{}'.format(begin, end)
                label = '{} - {}'.format(begin, end)
                opts.append((value, label))
            begin += 10
        return opts

    def queryset(self, request, queryset):
        try:
            if self.value():
                begin, end = self.value().split('-')
                if begin:
                    queryset = queryset.filter(birthdate__year__gte=int(begin))
                if end:
                    queryset = queryset.filter(birthdate__year__lte=int(end))
        except Exception as e:
            messages.error(request, str(e))
            queryset = SanctionEntity.objects.none()
        return queryset


class AddressCountryFilter(admin.SimpleListFilter):
    title = _('residence country')
    parameter_name = 'rcountry'

    def lookups(self, request, model_admin):
        opts = []
        for obj in Address.objects.distinct('country_iso2_code'):
            assert isinstance(obj, Address)
            opts.append((obj.country_iso2_code, obj.country_description))
        return opts

    def queryset(self, request, queryset):
        try:
            country = self.value()
            if country:
                queryset = SanctionEntity.objects.filter(address__country_iso2_code=country)
        except Exception as e:
            messages.error(request, str(e))
            queryset = SanctionEntity.objects.none()
        return queryset


class CitizenshipCountryFilter(admin.SimpleListFilter):
    title = _('citizenship country')
    parameter_name = 'ccountry'

    def lookups(self, request, model_admin):
        opts = []
        for obj in Citizenship.objects.distinct('country_iso2_code'):
            assert isinstance(obj, Citizenship)
            opts.append((obj.country_iso2_code, obj.country_description))
        return opts

    def queryset(self, request, queryset):
        try:
            country = self.value()
            if country:
                queryset = SanctionEntity.objects.filter(citizenship__country_iso2_code=country)
        except Exception as e:
            messages.error(request, str(e))
            queryset = SanctionEntity.objects.none()
        return queryset


class SanctionEntityAdmin(SanctionsListAdminBase):
    search_fields = (
        'namealias__whole_name__icontains',
        'eu_reference_number__iexact',
        'birthdate__year__iexact',
        'birthdate__birth_date__iexact',
    )
    inlines = (
        NameAliasInlineAdmin,
        AddressInlineAdmin,
        IdentificationInlineAdmin,
        CitizenshipInlineAdmin,
        BirthDateInlineAdmin,
        RemarkInlineAdmin,
        RegulationInlineAdmin,
    )
    raw_id_fields = (
        'source',
        'subject_type',
    )
    list_display = (
        'id',
        'source',
        'designation_details',
        'name_aliases',
        'birth_year',
        'united_nation_id',
        'eu_reference_number',
        'logical_id',
        'subject_type',
    )
    list_filter = (
        'subject_type',
        DecadeBornListFilter,
        AddressCountryFilter,
        CitizenshipCountryFilter,
    )

    def name_aliases(self, obj) -> str:
        assert isinstance(obj, SanctionEntity)
        all_names = ', '.join([e.whole_name for e in obj.namealias_set.all()])
        return all_names
    name_aliases.short_description = _('name aliases')  # type: ignore

    def birth_year(self, obj) -> str:
        assert isinstance(obj, SanctionEntity)
        return ', '.join([str(e.year) for e in obj.birthdate_set.all().distinct('year')])
    birth_year.short_description = _('birth year')  # type: ignore


admin.site.register(EuCombinedSanctionsList, EuCombinedSanctionsListAdmin)
admin.site.register(SubjectType, SubjectTypeAdmin)
admin.site.register(RegulationSummary, RegulationSummaryAdmin)
admin.site.register(SanctionEntity, SanctionEntityAdmin)
# admin.site.register(Regulation, SanctionEntitySubFieldAdminBase)
# admin.site.register(Remark, RemarkAdmin)
# admin.site.register(BirthDate, BirthDateAdmin)
# admin.site.register(Citizenship, CitizenshipAdmin)
# admin.site.register(NameAlias, NameAliasAdmin)
# admin.site.register(Identification, IdentificationAdmin)
# admin.site.register(Address, AddressAdmin)



# Generated by Django 2.2.5 on 2020-02-26 03:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('jsanctions', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='address',
            name='city',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='city'),
        ),
        migrations.AlterField(
            model_name='address',
            name='country_description',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='country description'),
        ),
        migrations.AlterField(
            model_name='address',
            name='place',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='place'),
        ),
        migrations.AlterField(
            model_name='address',
            name='po_box',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='p.o. box'),
        ),
        migrations.AlterField(
            model_name='address',
            name='region',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='region'),
        ),
        migrations.AlterField(
            model_name='address',
            name='street',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='street'),
        ),
        migrations.AlterField(
            model_name='address',
            name='zip_code',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='zip code'),
        ),
        migrations.AlterField(
            model_name='birthdate',
            name='city',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='city'),
        ),
        migrations.AlterField(
            model_name='birthdate',
            name='country_description',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='country description'),
        ),
        migrations.AlterField(
            model_name='birthdate',
            name='place',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='place'),
        ),
        migrations.AlterField(
            model_name='birthdate',
            name='region',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='region'),
        ),
        migrations.AlterField(
            model_name='birthdate',
            name='zip_code',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='zip code'),
        ),
        migrations.AlterField(
            model_name='citizenship',
            name='country_description',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='country description'),
        ),
        migrations.AlterField(
            model_name='citizenship',
            name='region',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='region'),
        ),
        migrations.AlterField(
            model_name='eucombinedsanctionslist',
            name='global_file_id',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='global file id'),
        ),
        migrations.AlterField(
            model_name='identification',
            name='country_description',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='country description'),
        ),
        migrations.AlterField(
            model_name='identification',
            name='identification_type_description',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='identification type code'),
        ),
        migrations.AlterField(
            model_name='identification',
            name='issued_by',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='issued by'),
        ),
        migrations.AlterField(
            model_name='identification',
            name='latin_number',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='latin number'),
        ),
        migrations.AlterField(
            model_name='identification',
            name='name_on_document',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='name on document'),
        ),
        migrations.AlterField(
            model_name='identification',
            name='number',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='number'),
        ),
        migrations.AlterField(
            model_name='identification',
            name='region',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='region'),
        ),
        migrations.AlterField(
            model_name='namealias',
            name='first_name',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='first name'),
        ),
        migrations.AlterField(
            model_name='namealias',
            name='function',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='function'),
        ),
        migrations.AlterField(
            model_name='namealias',
            name='last_name',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='last name'),
        ),
        migrations.AlterField(
            model_name='namealias',
            name='middle_name',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='middle name'),
        ),
        migrations.AlterField(
            model_name='namealias',
            name='title',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='title'),
        ),
        migrations.AlterField(
            model_name='namealias',
            name='whole_name',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='whole name'),
        ),
        migrations.AlterField(
            model_name='regulation',
            name='number_title',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='number title'),
        ),
        migrations.AlterField(
            model_name='regulation',
            name='organisation_type',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='organization type'),
        ),
        migrations.AlterField(
            model_name='regulation',
            name='programme',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='programmer'),
        ),
        migrations.AlterField(
            model_name='regulation',
            name='regulation_type',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='regulation type'),
        ),
        migrations.AlterField(
            model_name='regulationsummary',
            name='number_title',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='number title'),
        ),
        migrations.AlterField(
            model_name='sanctionentity',
            name='designation_details',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='designation details'),
        ),
        migrations.AlterField(
            model_name='sanctionentity',
            name='eu_reference_number',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='EU reference number'),
        ),
        migrations.AlterField(
            model_name='sanctionentity',
            name='united_nation_id',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='United Nation identifier'),
        ),
        migrations.AlterField(
            model_name='subjecttype',
            name='code',
            field=models.CharField(blank=True, default='', max_length=512, verbose_name='code'),
        ),
    ]
